 /*
 * Connections          - Arduino Pro Mini
 * ---------------------------------------
 * 
 * Receiver
 * Ext - (brown)        - GND
 * Ext + (red)          - Vin/RAW
 * Ext Sig (orange)     - TX
 * 
 * Drive battery
 * cell 1-              - GND
 * cell 1+              - R100k - A0 - R4k3 - GND (voltage divider max 26.7V -> 1.1V)
 * cell 2+              - R100k - A1 - R4k3 - GND (voltage divider max 26.7V -> 1.1V)
 * cell 3+              - R100k - A2 - R4k3 - GND (voltage divider max 26.7V -> 1.1V)
 * cell 4+              - R100k - A3 - R4k3 - GND (voltage divider max 26.7V -> 1.1V)
 * cell 5+              - R100k - A4 - R4k3 - GND (voltage divider max 26.7V -> 1.1V)
 * cell 6+              - R100k - A5 - R4k3 - GND (voltage divider max 26.7V -> 1.1V)
 */

#define VOLTAGE_SCALE 0.02608 // (100kOhm + 4.3kOhm) / 4.3kOhm * 1.1V / 1023
#define SAMPLING_PERIOD 1 // ms
#define FILTER_CUTOFF_FREQUENCY 10 // Hz

#include "JetiExProtocol.h"

JetiExProtocol jetiEx;

const int cellPin[] = {A0, A1, A2, A3, A4, A5};
int cells;
unsigned long timeSample; // us
float voltage[7], cellVoltage, cellVoltageMin, cellVoltageMinAll = 10; // V
int cellMin, cellMinAll;

JETISENSOR_CONST sensors[] PROGMEM = {
  //id  name              unit      data type               precision 0->0, 1->0.0, 2->0.00
  { 1,  "Vtot",           "V",      JetiSensor::TYPE_14b,   1},
  { 2,  "Vmin",           "V",      JetiSensor::TYPE_14b,   2},
  { 3,  "Cmin",           "",       JetiSensor::TYPE_14b,   0},
  { 4,  "Vmin all",       "V",      JetiSensor::TYPE_14b,   2},
  { 5,  "Cmin all",       "",       JetiSensor::TYPE_14b,   0},
  0 // end of array
};

void setup() {
  jetiEx.Start("Voltage Sensor", sensors);

  pinMode(LED_BUILTIN, OUTPUT);
  analogReference(INTERNAL);
  voltage[0] = 0;
  for (int cell = 1; cell <= 6; cell++) {
    voltage[cell] = VOLTAGE_SCALE * analogRead(cellPin[cell - 1]);
    if (voltage[cell] > 3.0) {
      cells = cell;
      digitalWrite(LED_BUILTIN, HIGH);
      delay(1000);
      digitalWrite(LED_BUILTIN, LOW);
      delay(1000);
    }
    else break;
  }
  
  timeSample = millis();
}

void loop() {
  timeSample += SAMPLING_PERIOD;
  digitalWrite(LED_BUILTIN, long(timeSample - millis()) <= 0); // switch led on if sampling period is too short
  while (long(timeSample - millis()) > 0); // wait for next sample
  
  for (int cell = 1; cell <= cells; cell++) {
    filter(voltage[cell], VOLTAGE_SCALE * analogRead(cellPin[cell - 1])); // V
    cellVoltage = voltage[cell] - voltage[cell - 1]; // V
    
    if (cell == 1) {
      cellVoltageMin = cellVoltage;
      cellMin = cell;
    }
    else if (cellVoltage < cellVoltageMin) {
      cellVoltageMin = cellVoltage;
      cellMin = cell;
    }
  }
  
  if (cellVoltageMin < cellVoltageMinAll) {
    cellVoltageMinAll = cellVoltageMin;
    cellMinAll = cellMin;
  }
  
  jetiEx.SetSensorValue(1, voltage[cells] * 10);
  jetiEx.SetSensorValue(2, cellVoltageMin * 100);
  jetiEx.SetSensorValue(3, cellMin);
  jetiEx.SetSensorValue(4, cellVoltageMinAll * 100);
  jetiEx.SetSensorValue(5, cellMinAll);
  
  jetiEx.DoJetiSend();
}

void filter(float &filteredValue, float newValue) {
  static float alpha = 2 * PI * 0.001 * SAMPLING_PERIOD * FILTER_CUTOFF_FREQUENCY / (2 * PI * 0.001 * SAMPLING_PERIOD * FILTER_CUTOFF_FREQUENCY + 1);
  static float beta = 1 - alpha;
  filteredValue = alpha * newValue + beta * filteredValue;
}
